%% Preservation.m derives the preservation matrix P
% This script gives us the output matrix P containing a set of preservation
% parameters used in the objective function of the Maximum Preservation
% Location Model - Homogenious Accidents(MPLM-HA). The matrix contains the 
% preservation parameter based on the first and total response times, 
% respectivly, given in minutes. The input parameters are i, j and k
% representing the demand node and the first and total response unit nodes.
% The parameter varies with the accident type a. 

clear all
close all
clc

%% Parameter declaration 
global A B accident_types demand_zones vehicle_zones vehicle_types  % Global variables 

% Taking in the calculated travel times for each vehicle type
T = load('TravelTimes.txt');

% Adding a time delay
%T = T+3;
% Number of demand zones and vehicle zones
[demand_zones,vehicle_zones] = size(T);

% Number of accident types 
%accidents = 4;                                 
accident_types = 2; % 1=fire, 2=traffic
vehicle_types = 3; 

%A = [0.12040, 0, 0.12040, 0];           % Exponential decreasing factor     
%B = [0.00150, 0.00850, 0, 0.00150];     % Linear decreasing factor 
A = [0.12040, 0];           % Exponential decreasing factor     
B = [0.00150, 0.00850];     % Linear decreasing factor 

% Declaring matrix size
P6D = zeros(demand_zones, accident_types, vehicle_zones, vehicle_types, vehicle_zones, vehicle_types);    
P4D = zeros(demand_zones, accident_types, vehicle_zones, vehicle_types);               

%% Calculations 
tic     % Iteration start
P1 = fopen('Preservation_last_response.txt','W');
P2 = fopen('Preservation_first_response.txt','W');

for a = 1:accident_types
    if a == 1 % Fire accident, requires first and second response
        for u = 1:vehicle_types
            for v = 1:vehicle_types
                
                for i = 1:demand_zones
                    for j = 1:vehicle_zones
                        for k = 1:vehicle_zones
                            % Assigning response times
                            if u == 1
                                tF = T(i,j); % First response, base unit
                            elseif u == 2
                                tF = 0.8 * T(i,j); % First response, small unit
                            elseif u == 3
                                tF = 1.2 * T(i,j); % First response, ladder unit
                            end % If
                            if v == 1
                                tL = T(i,k); % Total response, base unit 
                            elseif u == 2
                                tL = 0.8 * T(i,k); % First response, small unit
                            elseif u == 3
                                tF = 1.2 * T(i,k); % First response, ladder unit
                            end % If
                            if tF <= tL
                                % Using the P_function to calculate P(i,j,k,a)
                                P6D(i,a,j,u,k,v) = P_function_last_response(tF,tL,a); 
                                % Representing the values in a vector P
                                fprintf(P1,'(%d,%d,%d,%d,%d,%d)%d',i,a,j,u,k,v,P6D(i,a,j,u,k,v));
                            end % If
                        end % For
                    end % For
                end % For
            end % For
        end % For
    else % Other accidents, only require first response
        for u = 1:vehicle_types
            for i = 1:demand_zones
                for j = 1:vehicle_zones
                    % Assigning response times
                    if u == 1
                        t = T(i,j); % First response, base unit
                    elseif u == 2
                        t = 0.8 * T(i,j); % First response, small unit
                    elseif u == 3
                        t = 1.2 * T(i,j); % First response, ladder unit
                    end % If
                    % Using the P_function to calculate P(i,j,a)
                    P4D(i,a,j,u) = P_function_first_response(t,a); 
                    % Representing the values in a vector P
                    fprintf(P2,'(%d,%d,%d,%d)%d',i,a,j,u,P4D(i,a,j,u));
                end % For
            end % For
        end % For
    end % If
end % For
fclose('all');
toc     % Iteration stop